(function(global) { 'use strict'; define(async ({ // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
	'node_modules/web-ext-utils/browser/': { Menus, Commands, Windows, WebRequest, },
	'node_modules/web-ext-utils/utils/': { reportError, /*reportSuccess,*/ },
	'node_modules/web-ext-utils/update/': updated,
	'common/options': options,
	Tabs,
	require, module,
}) => { /* global setTimeout, */
let debug; options.debug.whenChange(([ value, ]) => { debug = value; });
debug && console.info('Ran updates', updated);

/**
 * Bugs in FF59:
 *  * [BUG] tabs can be put in a state { discarded: true, state: 'loading', } which they don't leave automatically (might only happening when discarding a tab that is about to load after sessionrestore)
 *  * [BAD] loading discarded tabs removes the favicon, which will be missing when the tab is discarded again before the favicon is restored
 *  * [API] favIconUrl can't be set
 *  * [BUG?] calling tabs.executeScript() for discarded tabs loads them
 *  * [BUG] calling tabs.executeScript() for never-loaded tabs only resolves after the tab is manually loaded (should reject or behave as if the tab was discarded)
 *  * [BUG?] never loaded tabs are not discarded (but internally pending)
 */


// keep a copy of the tab that last had its favicon removed (which is pointless as the favIconUrl cant be set -.-)
/*let lastRemovedFavicon = null; Tabs.onUpdated.addListener((id, change) => { // must run first
	if ('discarded' in change) { return; }
	lastRemovedFavicon = change.favIconUrl === null && syncTabs ? clone(Tabs.get(id)) : null;
	console.log('lastRemovedFavicon', lastRemovedFavicon, lastRemovedFavicon && lastRemovedFavicon.favIconUrl);
});*/


// only keep track of Tabs while options.onClose.value !== 'none'
let syncTabs = false; options.onClose.whenChange(([ value, ]) => {
	syncTabs = value !== 'none';
	Tabs.setEnabled(syncTabs); // must listen first
	[ onRemoved, onActivated, onUpdated, ].forEach(func =>
		Tabs[func.name][syncTabs ? 'addListener' : 'removeListener'](func)
	);
});


// add menus
Menus.create({
	title: 'Unload Tab',
	id: 'unloadTab',
	icons: { 64: 'icon.png', },
	contexts: [ 'tab', 'tools_menu', ],
});
Menus.create({
	title: 'Unload Other Tabs',
	id: 'unloadOtherTabs',
	icons: { 32: 'many.png', },
	contexts: [ 'tools_menu', ],
});


// respond to menu click
Menus.onClicked.addListener(async ({ menuItemId, }, { id, }) => { const tab = Tabs.get(id); try { switch (menuItemId) {
	case 'unloadTab': {
		if (tab.active) { const alt = syncTabs && findNext(tab); if (alt) {
			(await Tabs.update(alt.id, { active: true, }));
		} else {
			reportError('Not unloading', options.onClose.value !== 'none' ? 'No Tab to switch to' : 'Tab switching is disabled');
		} }
		(await Tabs.discard(tab.id));
	} break;
	case 'unloadOtherTabs': {
		(await Tabs.discard(Tabs.query({
			pinned: tab.pinned , discarded: false, windowId: tab.windowId,
		}).filter(_=>_.id !== tab.id).map(_=>_.id)));
	} break;
} } catch (error) { reportError(error); } });


// respond to (keyboard) commands
Commands && Commands.onCommand.addListener(async command => { console.log('command', command); try { switch (command) {
	case 'prevLoadedTab': (await seek(-1)); break;
	case 'nextLoadedTab': (await seek(+1)); break;
} } catch (error) { reportError(error); } });
async function seek(direction) {
	const window = (await Windows.getLastFocused({ windowTypes: [ 'normal', ], populate: !syncTabs, }));
	syncTabs && (window.tabs = Tabs.query({ windowId, }));
	const tab = window.tabs.find(_=>_.active);
	const tabs = window.tabs.sort((a, b) => a.index - b.index);
	console.log(clone(tabs), tab, start);

	function find(tab) { return tab && !tab.discarded && !tab.hidden && (alt = tab); }
	function increment(index) { return (index + direction + tabs.length) % tabs.length; }
	let alt; for ( // search in one direction, wrap around and return the original tab if no other is found
		let i = increment(tabs.indexOf(tab));
		!find(tabs[i]) && tabs[i] !== tab;
		i = increment(i)
	) { void 0; }

	console.log('tab, alt', tab, alt);
	alt && (await Tabs.update(alt.id, { active: true, }));
}


// respond to tab close
let activating = null;
async function onRemoved(id) { // choose the next active tab
	// console.log('closing', id, Tabs.get(id));
	const tab = Tabs.get(id); if (!tab.active) { return; }
	const alt = findNext(Tabs.get(id)); if (!alt) { return; }
	console.info('closing tab', id, ', activating', alt.id);
	activating = alt.id; setTimeout(() => activating === alt.id && (activating = null), 500);
	setTimeout(() => Tabs.update(alt.id, { active: true, }), 1000);
}
async function onActivated({ tabId: id, }) { // don't allow the wrong tab to be activated (shortly after closing)
	if (!activating || activating === id) { return; }
	console.warn('focusing wrong tab', id, clone(Tabs.get(id)));
	Tabs.update(activating, { active: true, });
	for (const time of [ 10, 35, 70, 120, 300, ]) { (await sleep(time));
		Tabs.update(activating, { active: true, }); console.info('force activate', id);
	}
}
async function onUpdated(id, change) { // don't allow tabs to load that are not active
	if (change.discarded !== false) { return; }
	const tab = Tabs.get(id); if (tab.active) { return; }
	// this also happens when legitimately focusing an unloaded tab (they won't be activated yet), but discarding won't have an effect
	console.info('background tab loads', id, clone(Tabs.get(id)));
	// const favIconUrl = lastRemovedFavicon && lastRemovedFavicon.id === id && lastRemovedFavicon.favIconUrl;
	Tabs.discard(id); tab.discarded = true; // so that on the close event (which happens after this one in FF59) this tab won't be selected
	for (const time of [ 10, 35, 70, 120, 300, ]) { (await sleep(time));
		Tabs.discard(id); console.info('force discard', id);
	}
	// !tab.favIconUrl && favIconUrl && Tabs.update(id, { favIconUrl, }); // restore favicon
}
// restoring tabs doesn't do any webRequests and webNavigation can't be canceled
/*WebRequest.onBeforeRequest.addListener( // don't allow tabs to load that are discarded
	({ tabId: id, url, }) =>  { console.warn('load', url); return Tabs.get(id).discarded ? ({ cancel: true, }) : null; },
	{ types: [ 'main_frame', ], urls: [ '<all_urls>', ] }, [ 'blocking', ],
);*/


// utils
function findNext(tab) { const { openerTabId, windowId, index, pinned, discarded, } = tab;
	console.log('findNext', ...arguments);
	let found = null; function find(tab) { return tab && !tab.discarded && !tab.hidden && (found = tab); }

	switch (options.onClose.value) {
		case 'none': return null; // shouldn't have called at all
		case 'prev': {
			if (find(Tabs.previous(windowId))) { console.log('prev'); return found; }
		} break;
		case 'right': {
			if (find(Tabs.find({ windowId, index: index + 1, }))) { console.log('right'); return found; }
		} break;
		case 'left': // happens automatically
	}

	const tabs = Tabs.query({ windowId, }).sort((a, b) => a.index - b.index);
	const start = tabs.indexOf(tab); if (start < 0) { return null; }

	console.log(clone(tabs), tab, start);

	for ( // search up and down at the same time. No need to wrap around
		let j = start - 1, i = start + 1, length = tabs.length;
		(j >= 0 || i < length) && !(find(tabs[j]) || find(tabs[i]));
		--j, ++i
	) { void 0; }
	return found;
}


// debug stuff
Object.assign(global, module.exports = {
	Browser: require('node_modules/web-ext-utils/browser/'),
	options,
});

function clone(arg) {
	return JSON.parse(JSON.stringify(arg));
}

function sleep(time) {
	return new Promise(done => setTimeout(done, time));
}

}); })(this);
